# Generated by Django 4.2.13 on 2024-07-10 12:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('openedx_ledger', '0011_deposit_models'),
    ]

    operations = [
        migrations.AlterField(
            model_name='deposit',
            name='sales_contract_reference_id',
            field=models.CharField(blank=True, db_index=True, help_text=('The reference ID for the specific sales contract which beget this deposit. The source system which ', 'stores the sales object is identified by the sales_contract_reference_provider field.'), max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='deposit',
            name='sales_contract_reference_provider',
            field=models.ForeignKey(blank=True, help_text='The system providing the source of truth for the sales contract which beget this deposit.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='deposits', to='openedx_ledger.salescontractreferenceprovider'),
        ),
        migrations.AlterField(
            model_name='historicaldeposit',
            name='sales_contract_reference_id',
            field=models.CharField(blank=True, db_index=True, help_text=('The reference ID for the specific sales contract which beget this deposit. The source system which ', 'stores the sales object is identified by the sales_contract_reference_provider field.'), max_length=255, null=True),
        ),
    ]
