# Generated by Django 3.2.23 on 2024-02-15 16:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openedx_ledger', '0009_add_email_and_title_to_transactions'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicaltransaction',
            name='parent_content_key',
            field=models.CharField(blank=True, db_index=True, help_text='Identifier for the parent of the content_key.  Also should be joinable with ContentMetadata.content_key in enterprise-catalog. In practice, this is the course key corresponding to a course run key.', max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='transaction',
            name='parent_content_key',
            field=models.CharField(blank=True, db_index=True, help_text='Identifier for the parent of the content_key.  Also should be joinable with ContentMetadata.content_key in enterprise-catalog. In practice, this is the course key corresponding to a course run key.', max_length=255, null=True),
        ),
    ]
