# Generated by Django 3.2.18 on 2023-10-24 19:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openedx_ledger', '0008_adjustment_model'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicaltransaction',
            name='content_title',
            field=models.CharField(blank=True, db_index=True, help_text='The title of the content associated with this Transaction.The title is captured at the time the Transaction is created and may not be up to date.', max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='historicaltransaction',
            name='lms_user_email',
            field=models.CharField(blank=True, db_index=True, help_text='The email of the Open edX LMS user record with which this Transaction is associated.The email is captured at the time the Transaction is created and may not be up to date.', max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='transaction',
            name='content_title',
            field=models.CharField(blank=True, db_index=True, help_text='The title of the content associated with this Transaction.The title is captured at the time the Transaction is created and may not be up to date.', max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='transaction',
            name='lms_user_email',
            field=models.CharField(blank=True, db_index=True, help_text='The email of the Open edX LMS user record with which this Transaction is associated.The email is captured at the time the Transaction is created and may not be up to date.', max_length=255, null=True),
        ),
    ]
