# Generated by Django 3.2.18 on 2023-03-13 18:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openedx_ledger', '0003_field_updates_20230216_1605'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalledger',
            name='idempotency_key',
            field=models.CharField(blank=True, db_index=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='ledger',
            name='idempotency_key',
            field=models.CharField(blank=True, max_length=255, unique=True),
        ),
    ]
