"""
Constants for the openedx_ledger app.
"""
CENTS_PER_US_DOLLAR = 100
LEDGER_DEFAULT_IDEMPOTENCY_KEY_PREFIX = 'ledger-default'
LEDGERED_SUBSIDY_IDEMPOTENCY_KEY_PREFIX = 'ledger-for-subsidy'

INITIAL_DEPOSIT_TRANSACTION_SLUG = 'initial-deposit'
