# Generated by Django 3.2.18 on 2023-05-01 08:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openedx_ledger', '0006_auto_20230404_1744'),
    ]

    operations = [
        migrations.AlterField(
            model_name='externalfulfillmentprovider',
            name='name',
            field=models.CharField(help_text='The name of the external reference type.', max_length=255),
        ),
    ]
